@@ -1,3 +1,7 @@
+# flake8: noqa
+# There's no way to ignore "F401 '...' imported but unused" warnings in this
+# module, but to preserve other warnings. So, don't check this module at all.
+
 #  Copyright 2021 The HuggingFace Team. All rights reserved.
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,9 +17,6 @@
 #  limitations under the License.
 
 from .ipu_configuration import IPUConfig
-from .models.bert import (
-    PipelinedBertForPretraining,
-    PipelinedBertForQuestionAnswering,
-)
+from .models.bert import PipelinedBertForPreTraining, PipelinedBertForQuestionAnswering
 from .trainer import IPUTrainer
 from .training_args import IPUTrainingArguments
